/* ***** BEGIN LICENCE BLOCK *****
 * Version: CePL 1.1
 * 
 * The contents of this file are subject to the Celtx Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.celtx.com/CePL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Celtx Script Manager.
 * 
 * The Initial Developer of the Original Code is Chad House and 4067479
 * Canada Inc. t/a CELTX.
 * 
 * Portions created by Chad House are Copyright (C) 2000-2004 Chad House,
 * parts created by Celtx are Copyright (C) 4067479 Canada Inc. All Rights
 * Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENCE BLOCK ***** */

var gApp = {
  QueryInterface: function QueryInterface (iid) {
    if (iid.equals(Components.interfaces.nsISupports) ||
        iid.equals(Components.interfaces.nsISupportsWeakReference) ||
        iid.equals(Components.interfaces.nsIController) ||
        iid.equals(Components.interfaces.nsIObserver))
      return this;
    throw Components.results.NS_NOINTERFACE;
  },


  stringBundle: null,
  initialized: false,


  getText: function (name, params) {
    if (! this.sb) {
      var bundleSvc = getStringBundleService();
      var bundlePath = Cx.LOCALE_PATH + "celtx.properties";
      this.sb = bundleSvc.createBundle(bundlePath);
    }
    try {
      if (params)
        return this.sb.formatStringFromName(name, params, params.length);
      else
        return this.sb.GetStringFromName(name);
    }
    catch (ex) {
      dump("*** gApp.getText: Couldn't get " + name + "\n");
      return null;
    }
  },


  commands: {
    "cmd-new-project": 1,
    "cmd-open-project": 1,
    "cmd-download-project": 1,
    "cmd-about-celtx": 1,
    "cmd-help-support": 1,
    "cmd-celtx-wiki": 1,
    "cmd-video-tutorial": 1,
    "cmd-help-report-bug": 1,
    "cmd-help-account": 1
  },


  supportsCommand: function (cmd) {
    return this.commands[cmd] == 1;
  },


  isCommandEnabled: function (cmd) {
    switch (cmd) {
      default:
        return true;
    }
  },


  doCommand: function (cmd) {
    if (! this.initialized)
      this.init();
    switch (cmd) {
      case "cmd-new-project":
        this.cmdNewProject();
        break;
      case "cmd-open-project":
        this.openProject();
        break;
      case "cmd-download-project":
        this.cmdDownloadProject();
        break;
      case "cmd-about-celtx":
        this.showAboutDialog();
        break;
      case "cmd-help-support":
        this.openBrowser(Cx.FORUMS_URL);
        break;
      case "cmd-celtx-wiki":
        this.openBrowser(Cx.USER_GUIDE_URL);
        break;
      case "cmd-video-tutorial":
        this.openBrowser(Cx.WALKTHRU_URL);
        break;
      case "cmd-help-report-bug":
        this.openBrowser(Cx.BUG_REPORT_URL);
        break;
      case "cmd-help-account":
        this.cmdAccountWizard();
        break;
    }
  },


  setToolbarMode: function setToolbarMode (mode) {
    var ps = getPrefService().getBranch("celtx.toolbar.");
    ps.setCharPref("show", mode);
  },


  observe: function (subject, topic, data) {
    if (topic != "nsPref:changed" || data != "celtx.toolbar.show")
      return;
    var ps = getPrefService().getBranch("celtx.toolbar.");
    var mode = ps.getCharPref("show");
    var obs = document.getElementById("toolbar-broadcaster");
    if (obs)
      obs.setAttribute("mode", mode);
    var menuitem = document.getElementById("menu-toolbarmode-" + mode);
    if (! menuitem) {
      return;
    }
    if (menuitem && menuitem.getAttribute("checked") != "true")
      menuitem.setAttribute("checked", "true");
    else
      dump("*** gApp.observe: menuitem already marked?\n");
  },


  handleEvent: function handleEvent (event) {
    dump("--- gApp.handleEvent\n");
    if (event.type == "load")
      this.init();
    else
      dump("*** gApp.handleEvent: Unknown event '" + event.type + "'\n");
  },


  get userID () {
    try {
      prefsvc = getPrefService().QueryInterface(
        Components.interfaces.nsIPrefBranch);
      return prefsvc.getCharPref(Cx.PREF_USER_ID);
    }
    catch (ex) {
      return null;
    }
  },


  set userID (val) {
    try {
      prefsvc = getPrefService().QueryInterface(
        Components.interfaces.nsIPrefBranch);
      if (val)
        prefsvc.setCharPref(Cx.PREF_USER_ID, val);
      else
        prefsvc.clearUserPref(Cx.PREF_USER_ID);
    }
    catch (ex) {
      dump("*** gApp.userID setter: " + ex + "\n");
    }
  },


  init: function init () {
    if (this.initialized) return;

    var ps = getPrefService();
    ps = ps.QueryInterface(Components.interfaces.nsIPrefBranch2);
    ps.addObserver("celtx.toolbar.", this, false);
    this.initPrefs();
    rebuildRecentProjectsMenu();
    this.checkTemplates();
    this.checkSamples();
    this.initialized = true;
  },


  checkProfileDir: function checkProfileDir (dirname) {
    // See if there's already a directory with at least
    // one entry in it. If so, return without action.
    var IFile = Components.interfaces.nsIFile;
    var dstdir = currentProfileDir();
    dstdir.append(dirname);
    if (dstdir.exists()) {
      var entries = dstdir.directoryEntries;
      while (entries.hasMoreElements()) {
        var entry = entries.getNext().QueryInterface(IFile);
        if (entry.leafName.match(/\.t?celtx$/))
          return;
      }
    }
    else {
      dstdir.create(1, dstdir.parent.permissions);
    }

    // Copy over all the default templates
    var srcdir = currentProcessDir();
    srcdir.append("defaults");
    srcdir.append("profile");
    srcdir.append(dirname);
    var srcfiles = srcdir.directoryEntries;
    while (srcfiles.hasMoreElements()) {
      var srcfile = srcfiles.getNext().QueryInterface(IFile);
      var dstfile = dstdir.clone();
      dstfile.append(srcfile.leafName);
      if (dstfile.exists() && dstfile.fileSize != srcfile.fileSize)
        dstfile.remove(false);
      if (! dstfile.exists()) {
        try {
          srcfile.copyTo(dstdir, srcfile.leafName);
        }
        catch (ex) {
          dump("*** checkProfileDir: " + ex + "\n");
        }
      }
    }
  },


  checkTemplates: function checkTemplates () {
    this.checkProfileDir(Cx.TEMPLATES_DIR);
  },


  checkSamples: function checkSamples () {
    this.checkProfileDir(Cx.SAMPLES_DIR);
  },


  initPrefs: function initPrefs () {
    var prefsvc = null;
    try {
      prefsvc = getPrefService().QueryInterface(
        Components.interfaces.nsIPrefBranch);
    }
    catch (ex) {
      throw "Unable to get the preferences service";
    }

    try {
      var clearpass = prefsvc.getCharPref("celtx.user.password");
      if (clearpass) {
        prefsvc.setCharPref("celtx.user.encpassword",
          base64_encodew(clearpass));
        prefsvc.clearUserPref("celtx.user.password");
      }
    }
    catch (ex) {}

    var firstLaunch = true;
    try {
      firstLaunch = prefsvc.getBoolPref("celtx.firstlaunch");
      if (firstLaunch) {
        prefsvc.setBoolPref("celtx.firstlaunch", false);
        firstLaunch = false;
      }
    }
    catch (ex) {
      prefsvc.setBoolPref("celtx.firstlaunch", true);
    }

    var lastVersion = new Version();
    try {
      lastVersion = new Version(prefsvc.getCharPref("celtx.version"));
    }
    catch (ex) {}

    
    var pre097 = (lastVersion.compare(new Version("0.9.7")) < 0);
    var pre098 = (lastVersion.compare(new Version("0.9.8")) < 0);
    var pre099 = (lastVersion.compare(new Version("0.9.9")) < 0);

    if (pre098) {
      try {
        this.createDefaultDeptList();
      }
      catch (ex) {
        dump("*** createDefaultDeptList: " + ex + "\n");
      }
    }

    if (pre099) {
      try {
        if (prefsvc.prefHasUserValue("print.macosx.pagesetup-2"))
          prefsvc.clearUserPref("print.macosx.pagesetup-2");
      }
      catch (ex) {
        dump("*** pre099 update: " + ex + "\n");
      }
    }

    prefsvc.setCharPref("celtx.version", Cx.VERSION);

    // Make sure the user dictionary has a value, otherwise inline spell
    // checking will fail silently.
    try {
      prefsvc.getCharPref("spellchecker.dictionary");
    }
    catch (ex) {
      prefsvc.setCharPref("spellchecker.dictionary", "en-US");
    }

    // Set the toolbar icon/text mode
    try {
      this.setToolbarMode(prefsvc.getCharPref("celtx.toolbar.show"));
    }
    catch (ex) {
      this.setToolbarMode("icons");
    }
    // No event is sent if the preference doesn't actually change, so we need
    // to force it to set the toolbar mode and check the menu item on init.
    this.observe(prefsvc, "nsPref:changed", "celtx.toolbar.show");

    try {
      var dir = prefsvc.getCharPref("celtx.projectsdirectory");
    }
    catch (ex) {
      try {
        dir = userDocsDir().path;
        prefsvc.setCharPref("celtx.projectsdirectory", dir);
      }
      catch (ex2) {
        dump("*** Unable to set projects directory: " + ex2 + "\n");
      }
    }

    try {
      gApp.resetPrintingPrefs();
    }
    catch (ex) {
      dump("*** resetPrintingPrefs: " + ex + "\n");
    }
  },


  createDefaultDeptList: function createDefaultDeptList () {
    var rdfsrc = currentProfileDir();
    rdfsrc.append(Cx.PREFS_FILE);
    if (rdfsrc.exists())
      return;

    // serializeDataSourceToFile(getInMemoryDataSource(), rdfsrc);

    var rdfsvc = getRDFService();
    // var ds = rdfsvc.GetDataSourceBlocking(fileToFileURL(rdfsrc));
    var ds = getInMemoryDataSource();
    var seqres = rdfsvc.GetResource(Cx.NS_CX + "Prefs/Categories");
    var seq = new RDFSeq(ds, seqres);

    // var itemlist = [ "Location", "Cast", "Extras", "Hair", "Makeup",
    //   "Wardrobe", "Props", "Set", "SetDressing" ];

    var schemads = rdfsvc.GetDataSourceBlocking(Cx.SCHEMA_URL);
    var defaultres = rdfsvc.GetResource(Cx.SCHEMA_URL + "#default-markup");
    var defaultseq = new RDFSeq(schemads, defaultres);
    var itemlist = defaultseq.toArray();

    for (var i = 0; i < itemlist.length; ++i) {
      // seq.push(rdfsvc.GetResource(Cx.NS_CX + itemlist[i]));
      seq.push(itemlist[i]);
    }
    try {
      // var rds = ds.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
      // rds.Flush();
      serializeDataSourceToFile(ds, rdfsrc);
    }
    catch (ex) {
      dump("*** createDefaultDeptList: " + ex + "\n");
    }
  },


  setPrintMargins: function setPrintMargins (top, right, bottom, left) {
    var PSSVC = Components.classes["@mozilla.org/gfx/printsettings-service;1"]
      .getService(Components.interfaces.nsIPrintSettingsService);
    var opts = PSSVC.QueryInterface(Components.interfaces.nsIPrintOptions);
    if (! opts.availablePrinters().hasMoreElements()) {
      dump("--- setPrintMargins: No printers available. Aborting.\n");
      return;
    }

    try {
      var printSettings = PrintUtils.getPrintSettings();
      printSettings.marginTop       = top;
      printSettings.marginBottom    = bottom;
      printSettings.marginLeft      = left;
      printSettings.marginRight     = right;
      var flags = printSettings.kInitSaveMargins;
      PSSVC.savePrintSettingsToPrefs(printSettings, true, flags);
    }
    catch (ex) {
      dump("*** app.setPrintMargins: " + ex + "\n");
    }
  },


  resetPrintingPrefs: function resetPrintingPrefs (showPageNumbers) {
    // Oops, this should never have been set.
    var IPrefBranch = Components.interfaces.nsIPrefBranch;
    var prefsvc = getPrefService().QueryInterface(IPrefBranch);
    if (prefsvc.prefHasUserValue("print.print_printer") &&
        prefsvc.getCharPref("print.print_printer") != "") {
      prefsvc.clearUserPref("print.print_printer", "");
    }

    var PSSVC = Components.classes["@mozilla.org/gfx/printsettings-service;1"]
      .getService(Components.interfaces.nsIPrintSettingsService);
    var opts = PSSVC.QueryInterface(Components.interfaces.nsIPrintOptions);
    if (! opts.availablePrinters().hasMoreElements()) {
      dump("--- resetPrintingPrefs: No printers available. Aborting.\n");
      return;
    }
    // Initialise the printer settings to our preferred defaults
    try {
      var printSettings = PrintUtils.getPrintSettings();
      printSettings.headerStrLeft   = "";
      printSettings.headerStrCenter = "";
      printSettings.headerStrRight  = showPageNumbers ? "&P" : "";
      printSettings.footerStrLeft   = "";
      printSettings.footerStrCenter = "";
      printSettings.footerStrRight  = "";
      printSettings.printBGColors   = true;
      printSettings.marginTop       = 0.5;
      printSettings.marginBottom    = 0.5;
      printSettings.marginLeft      = 0.5;
      printSettings.marginRight     = 0.5;
      printSettings.shrinkToFit     = false;
      var flags = printSettings.kInitSaveHeaderLeft   |
        printSettings.kInitSaveHeaderCenter |
        printSettings.kInitSaveHeaderRight  |
        printSettings.kInitSaveFooterLeft   |
        printSettings.kInitSaveFooterCenter |
        printSettings.kInitSaveFooterRight  |
        printSettings.kInitSaveBGColors     |
        printSettings.kInitSaveMargins      |
        printSettings.kInitSaveShrinkToFit  ;
      PSSVC.savePrintSettingsToPrefs(printSettings, true, flags);
    }
    catch (ex) {
      dump("*** app.initPrefs: " + ex + "\n");
    }
  },


  cmdNewProject: function cmdNewProject () {
    try {
      if (0) {
        window.openDialog(Cx.CONTENT_PATH, "_blank", Cx.NEW_WINDOW_FLAGS);
      }
      else if (0) {
        var spec = this.createTempProject();
        window.openDialog(Cx.CONTENT_PATH, "_blank", Cx.NEW_WINDOW_FLAGS, spec);
      }
      else {
        window.openDialog(Cx.CONTENT_PATH + "newproject.xul", "_blank",
          Cx.NEW_WINDOW_FLAGS + ",centerscreen");
      }
    }
    catch (ex) {
      dump("*** cmdNewProject: " + ex + "\n");
    }
  },


  findProjectWindowByID: function findProjectWindowByID (id) {
    var wm = getWindowMediator();
    var e  = wm.getEnumerator('celtx:main');
    while (e.hasMoreElements()) {
      var w = e.getNext();
      if (w.gProject && w.gProject.id == id) {
        return w;
      }
    }

    return null;
  },


  findProjectWindowByFile: function findProjectWindowByFile (file) {
    var tmpfile = getTempDir().clone();
    var projwin = null;
    tmpfile.append("project.rdf");
    tmpfile.createUnique(0, 0644);
    var reader = getZipReader();
    try {
      reader.init(file);
      reader.open();
      var entry = reader.getEntry("project.rdf");
      if (! entry)
        throw "no project.rdf in project archive";
      reader.extract("project.rdf", tmpfile);
      var project = new Project(fileToFileURL(tmpfile));
      projwin = this.findProjectWindowByID(project.id);
    }
    catch (ex) {
      dump("*** findProjectWindowByFile: " + ex + "\n");
    }

    try {
      if (tmpfile.exists())
        tmpfile.remove(false);
    }
    catch (ex) {}

    try {
      reader.close();
    }
    catch (ex) {}

    return projwin;
  },


  // Obsolete method.
  findProjectWindowByPath: function findProjectWindowByPath (path) {
    var wm = getWindowMediator();
    var e  = wm.getEnumerator('celtx:main');
    while (e.hasMoreElements()) {
      var w = e.getNext();
      if (w.gProject && w.gProject.projectFolder.path == path) {
        return w;
      }
    }

    return null;
  },


  createTempProject: function createTempProject () {
    var projdir = getTempDir();
    projdir.append("project");
    projdir.createUnique(1, projdir.parent.permissions);
    var projfile = gProjMgr.createProject(projdir);
    var project = new Project(projfile);
    project.init();
    project.flush();
    return projfile;
  },


  createProject: function createProject (dir, title, docs) {
    var rdfsvc = getRDFService();
    var doctypeurl = Cx.CONTENT_PATH + "doctypes.rdf";
    var doctyperdf = rdfsvc.GetDataSourceBlocking(doctypeurl);
    var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");

    var projdir = dir.clone();
    projdir.append(sanitizeFilename(title));
    projdir.createUnique(1, 0700);
    var celtxfile = projdir.clone();
    celtxfile.append(projdir.leafName + ".celtx");
    celtxfile.create(0, 0600);

    var projFile = gProjMgr.createProject(projdir, title);
    var project = new Project(projFile);
    project.init();
    gProjMgr.setSaveLocation(project, fileToFileURL(celtxfile));
    for (var i = 0; i < docs.length; i++) {
      var doctype = rdfsvc.GetResource(docs[i]);
      var doctitle = doctyperdf.GetTarget(doctype, titlearc, true);
      if (doctitle) {
        doctitle = doctitle.QueryInterface(Components.interfaces.nsIRDFLiteral);
        project.createDocument(doctitle.Value, doctype);
      }
      else
        dump("*** no doctitle for " + doctype.Value + "\n");
    }
    project.flush();

    return projFile;
  },


  openProject: function openProject () {
    var IFilePicker = Components.interfaces.nsIFilePicker;
    var fp = getFilePicker();
    fp.init(window, gApp.getText("OpenProject"), IFilePicker.modeOpen);
    fp.appendFilter(gApp.getText("CeltxProject"), "*.celtx");
    // fp.appendFilter(gApp.getText("CeltxTemplate"), "*.tceltx");
    fp.displayDirectory = getCeltxProjectsDir();
    if (fp.show() != IFilePicker.returnOK)
      return false;

    if (! (fp.file.leafName.match(/\.t?celtx$/))) {
      alert(gApp.getText("InvalidProjectFile"));
      return false;
    }
    var spec = fp.fileURL.spec;

    setCeltxProjectsDir(fp.file.parent);

    // Check if the project is already open
    var existingWindow = this.findProjectWindowByFile(fp.file);
    if (existingWindow) {
      existingWindow.focus();
      return true;
    }

    window.openDialog(Cx.CONTENT_PATH, "_blank", Cx.NEW_WINDOW_FLAGS, spec);

    return true;
  },


  cmdDownloadProject: function cmdDownloadProject () {
    return downloadProject();
  },


  showAboutDialog: function showAboutDialog () {
    window.openDialog(Cx.CONTENT_PATH + "about.xul", "_blank",
                      Cx.MODAL_DIALOG_FLAGS);
  },


  showPicklist: function showPicklist (filter, obj) {
    window.openDialog(Cx.CONTENT_PATH + "picklist.xul", "_blank",
                      Cx.MODAL_DIALOG_FLAGS, filter, obj, gProject);
  },


  openBrowser: function openBrowser (url) {
    var ios = getIOService();
    var eps = getExternalProtocolService();
    var uri = ios.newURI(url, null, null);
    if (uri.scheme != "http" && uri.scheme != "https") {
      dump("*** openBrowser only supports http[s] uris!\n");
      return;
    }
    if (! eps.externalProtocolHandlerExists(uri.scheme)) {
      dump("*** No external handler for " + uri.scheme + "\n");
      return;
    }
    eps.loadURI(uri, null);
  },


  openContactsWindow: function openContactsWindow () {
    window.openDialog(Cx.CONTENT_PATH + "contacts.xul", "contacts",
                      Cx.NEW_WINDOW_FLAGS);
  },


  authenticatedForService: function authenticatedForService (service) {
    var authMgr = getAuthManager();

    // Check if already auth'd, callback true if so
    var rv = { domain: {}, username: {}, password: {} };

    try {
      // Throws an exception if not auth'd
      authMgr.getAuthIdentity(Cx.AUTH_SCHEME, Cx.PUBLISH_SERVER, Cx.AUTH_PORT,
                              Cx.AUTH_TYPE, Cx.AUTH_REALM, Cx.AUTH_PATH,
                              rv.domain, rv.username, rv.password);
      return true;
    }
    catch (ex) {
      return false;
    }
  },


  logout: function () {
    // There's no scriptable access to clearAuthEntry, so we just have
    // to clear it all.
    var authMgr = getAuthManager();
    authMgr.clearAll();
    // We need some sort of notifier for login/logout events.
  },


  authenticate: function authenticate (service, callback, reattempt) {
    dump("--- gApp.authenticate called by " + Components.stack.caller + "\n");

    getCeltxService().login(service,
      { onLogin: function (success) { callback(success); } },
      reattempt, window);
    return;

    // XXX: OBSOLETE

    var authMgr = getAuthManager();

    if (this.authenticatedForService(service)) {
      // XXX maybe test-auth with current credentials anyway?
      dump("already auth'd\n");
      // dump("rv.username.value: " + rv.username.value + "\n");
      callback(true);
      return;
    }

    /*
    if (! (this.userID || reattempt)) {
      var reg = this.getRegistration(true);
      if (! reg) {
        dump("*** Registration canceled.\n");
        callback(false);
        return;
      }
      dump("*** Authenticating as " + reg.username + "/" + reg.password + "\n");
      this.authenticateAs(reg.username, reg.password, service, callback);
      return;
    }
    */
    /*
    // Not auth'd yet
    var auth = { username: '', password: '', service: service,
                 server: Cx.PUBLISH_SERVER };
    try {
      auth.username = getPrefService().getCharPref(Cx.PREF_USER_ID);
    }
    catch (ex) {}

    window.openDialog(Cx.CONTENT_PATH + "authenticate.xul",
                      "_blank",
                      Cx.MODAL_DIALOG_FLAGS,
                      auth);

    if (auth.canceled) {
      dump("canceled\n");
      callback(false);
      return;
    }

    authMgr.setAuthIdentity(Cx.AUTH_SCHEME, Cx.PUBLISH_SERVER, Cx.AUTH_PORT,
                            Cx.AUTH_TYPE, Cx.AUTH_REALM, Cx.AUTH_PATH,
                            Cx.AUTH_DOMAIN, auth.username, auth.password);
    */
    // XXX deadman timer?
    var request = new XMLHttpRequest();
    request.onload  = function () { authLoaded(request, service, callback) };
    request.onerror = function () { authError (request, service, callback) };
    request.open("GET", Cx.AUTH_URL);
    var channel = request.channel;
    var prompt = new CeltxAuthPrompt(service, channel.notificationCallbacks,
                                     request);
    if (reattempt)
      prompt.reattempt = true;
    channel.notificationCallbacks = prompt;
    request.send(null);
  },


  authenticateAs: function authenticateAs (username, password,
                                           service, callback,
                                           synchronous) {
    dump("--- gApp.authenticateAs called by " + Components.stack.caller + "\n");

    getCeltxService().loginAs(username, password, service,
      { onLogin: function (success) { callback(success); } }, synchronous);
    return;

    // XXX: OBSOLETE

    var authMgr = getAuthManager();
    authMgr.setAuthIdentity(Cx.AUTH_SCHEME, Cx.PUBLISH_SERVER, Cx.AUTH_PORT,
                            Cx.AUTH_TYPE, Cx.AUTH_REALM, Cx.AUTH_PATH,
                            Cx.AUTH_DOMAIN, username, password);

    var request = new XMLHttpRequest();
    if (! synchronous) {
      request.onload  = function () {
        callback(request.status == 200);
      };
      request.onerror = function () {
        callback(false);
      };
    }
    request.open("GET", Cx.AUTH_URL, ! synchronous);
    var channel = request.channel;
    var prompt = new SuppressedAuthPrompt(channel.notificationCallbacks);
    channel.notificationCallbacks = prompt;
    request.send(null);
    if (synchronous)
      callback(request.status == 200);
  },


  cmdAccountWizard: function cmdAccountWizard () {
    this.getRegistration();
  },


  getRegistration: function getRegistration (showWarning) {
    var reg = {
      name:     "",
      email:    "",
      prefix:   Cx.REGISTER_URL,
      canceled: false,
      existing: false,
      warning: showWarning
    };

    window.openDialog(Cx.CONTENT_PATH + "regwizard.xul",
                      "registrationwizard",
                      isMac() ? "chrome,modal,centerscreen,dialog=no,titlebar"
                              : "chrome,modal,centerscreen",
                      reg);

    if (reg.canceled) {
      return null;
    }

    try {
      const IPrefBranch = Components.interfaces.nsIPrefBranch;
      var prefs = getPrefService().QueryInterface(IPrefBranch);
      prefs.setCharPref(Cx.PREF_USER_ID, reg.username);
      this.userID = reg.username;
    }
    catch (ex) {
      dump("Registration failed: unable to store user ID pref\n");
    }

    return reg;
  }
};


var gProjMgr = new ProjectManager();

window.controllers.appendController(gApp);


function addToRecentProjects (project) {
  var recent = gProjMgr.recentProjects;
  recent.remove(project.res);
  recent.insert(project.res, 0);
  while (recent.length > 10)
    recent.remove(recent.length - 1);
  gProjMgr.flush();
  getObserverService().notifyObservers(gProjMgr,
    "celtx:recent-projects-changed", null);
}


function removeFromRecentProjects (projres) {
  gProjMgr.recentProjects.remove(projres);
}


function rebuildRecentProjectsMenu () {
  var rdfsvc = getRDFService();
  var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
  var menu = document.getElementById("menu-recent-projects");
  var tbpopup = document.getElementById("recent-popup");

  // Check for the menu, but don't require the toolbar popup, since
  // hiddenWindow.xul has a menu but no toolbar.
  if (! menu)
    return;

  while (menu.hasChildNodes())
    menu.removeChild(menu.lastChild);
  if (tbpopup) {
    while (tbpopup.hasChildNodes())
      tbpopup.removeChild(tbpopup.lastChild);
  }
  var projects = gProjMgr.recentProjects.toArray();
  for (var i = 0; i < projects.length; i++) {
    try {
      var proj = projects[i].QueryInterface(
        Components.interfaces.nsIRDFResource);
      var item = document.createElement("menuitem");
      item.setAttribute("value", proj.Value);
  
      var projuri = gProjMgr.getSaveLocation({ res: proj });
      if (! projuri)
        continue;

      var projfile = fileURLToFile(projuri);
      if (projfile && projfile.leafName.match(/\.celtx$/)) {
        item.setAttribute("label", projfile.leafName.replace(/\.celtx$/, ""));
        item.setAttribute("fileuri", projuri);
      }
      else
        continue;

      item.setAttribute("oncommand",
        "openRecentProject(this); event.stopPropagation();");
      menu.appendChild(item);
      if (tbpopup)
        tbpopup.appendChild(item.cloneNode(true));
    }
    catch (ex) {
      dump("*** rebuildRecentProjectsMenu: " + ex + "\n");
    }
  }
}


// This is really just a helper for the splash page
function getRecentProjectList () {
  var recent = gProjMgr.recentProjects.toArray();
  var projs = [];
  var ILit = Components.interfaces.nsIRDFLiteral;
  var rdfsvc = getRDFService();
  var titlearc = rdfsvc.GetResource(Cx.NS_DC + "title");
  var descarc = rdfsvc.GetResource(Cx.NS_DC + "description");
  for (var i = 0; i < recent.length; i++) {
    var proj = {};
    var res = recent[i].QueryInterface(Components.interfaces.nsIRDFResource);
    proj.res = res.Value;
    var title = getRDFString(gProjMgr.ds, res, titlearc);
    proj.title = title ? title : gApp.getText("Untitled");
    proj.desc = getRDFString(gProjMgr.ds, res, descarc);
    projs.push(proj);
  }
  return projs;
}


function openProjectByRes (resuri) {
  var id = resuri.replace(/^.*\//, '');
  var existingWindow = gApp.findProjectWindowByID(id);
  if (existingWindow && ! existingWindow.closed) {
    existingWindow.focus();
    return;
  }
  var rdfsvc = getRDFService();
  var projres = rdfsvc.GetResource(resuri);
  // Definitely prone to breakage. getSaveLocation shouldn't take a project
  // as argument
  var fileuri = gProjMgr.getSaveLocation({ res: projres });
  var file = fileURLToFile(fileuri);
  if (fileuri && file.exists()) {
    window.openDialog(Cx.CONTENT_PATH, "_blank", Cx.NEW_WINDOW_FLAGS, fileuri);
  }
  else {
    // Kick it off to a regular file open dialog
    var title = gApp.getText("ProjectNotFoundTitle");
    var msg = gApp.getText("ProjectNotFoundMsg", [ file.path ]);
    var ps = getPromptService();
    ps.alert(window, title, msg);
    gApp.openProject();
  }
}


function openRecentProject (menuitem) {
  var rdfsvc = getRDFService();
  var projres = rdfsvc.GetResource(menuitem.getAttribute("value"));
  var fileuri = menuitem.getAttribute("fileuri");
  if (! fileuri)
    fileuri = gProjMgr.getSaveLocation({ res: projres });

  var file = fileURLToFile(fileuri);
  try {
    if (! file)
      throw "Invalid file URI: " + fileuri;
    if (! file.exists())
      throw "File not found";
    var existingWindow = gApp.findProjectWindowByFile(file);
    if (existingWindow && ! existingWindow.closed) {
      existingWindow.focus();
      return;
    }
    window.openDialog(Cx.CONTENT_PATH, "_blank", Cx.NEW_WINDOW_FLAGS, fileuri);
  }
  catch (ex) {
    var title = gApp.getText("ProjectNotFoundTitle");
    var msg = gApp.getText("ProjectNotFoundMsg",
      [ file ? file.path : fileuri ]);
    getPromptService().alert(window, title, msg);
    removeFromRecentProjects(projres);
  }
}


function downloadProject () {
  if (! gApp.authenticatedForService("get-project")) {
    var callback = function (succeeded) {
      if (succeeded) goDoCommand("cmd-download-project");
    };
    gApp.authenticate("get-project", callback);
    return false;
  }

  var config = { manager: gProjMgr,
                 projURL: Cx.PROJ_LIST_URL,
                 authURL: Cx.AUTH_URL
  };
    
  window.openDialog(Cx.CONTENT_PATH + 'getproj.xul',
                    '_blank',
                    Cx.RESIZABLE_DIALOG_FLAGS,
                    config);

  if (config.canceled) return false;

  dump("project: " + config.projectURL + "\ncontents: " + config.contentsURL + "\n");

  try {
    var id = config.projectURL.replace(/^.*\//, '');
    if (gApp.findProjectWindowByID(id)) {
      var title = gApp.getText("NoDownloadTitle");
      var msg = gApp.getText("NoDownloadProjectOpenMsg");
      var ps = getPromptService();
      ps.alert(window, title, msg);
      return false;
    }
  }
  catch (ex) {
    dump("*** downloadProject: " + ex + "\n");
  }

  var dir = getTempDir();
  dir.append("project");
  dir.createUnique(1, dir.parent.permissions);  

  var syncArg = {
    path:      dir,
    fetchURL:  config.contentsURL,
    projURL:   config.projectURL
  };

  window.openDialog(Cx.CONTENT_PATH + 'sync.xul', '_blank',
                    Cx.MODAL_DIALOG_FLAGS, 'download', syncArg);

  if (! syncArg.succeeded) {
    celtxBugAlert(gApp.getText("DownloadFailed"));
    return false;
  }

  var projectFile = dir.clone();
  projectFile.append(Cx.PROJECT_FILE);
  if (! projectFile.exists()) {
    celtxBugAlert(gApp.getText("ProjectIncomplete"));
    return false;
  }

  window.openDialog(Cx.CONTENT_PATH, "_blank", Cx.NEW_WINDOW_FLAGS,
                    fileToFileURL(projectFile));

  return true;
}


function findDirForDownload (uri, title) {
  var m = new RDFModel(gProjMgr.ds);
  var projectParent = getCeltxProjectsDir();
  var pathRes = m.target(RES(uri), PROP('cx:localpath'));
  if (pathRes) {
    var path = fileURLToFile(pathRes.value);
    if (path.exists()) {
      // return path.parent;
      projectParent = path.parent;
    }
  }

  while (true) {
    // Otherwise, ask for a location
    var saveConfig = {
      accepted: false,
      title: title,
      location: getCeltxProjectsDir(),
      docs: [],
      hideAdvanced: true
    };

    window.openDialog(Cx.CONTENT_PATH + "newproject.xul", "_blank",
                      Cx.MODAL_DIALOG_FLAGS, saveConfig);
    if (! saveConfig.accepted) return;

    var projdir = saveConfig.location.clone();
    projdir.append(sanitizeFilename(saveConfig.title));
    if (projdir.exists()) {
      var projfile = projdir.clone();
      projfile.append(Cx.PROJECT_FILE);
      // If there isn't a project.rdf in the destination directory, there's
      // a good chance the user just doesn't realise their action would
      // cause a name conflict with a non-project folder. Do NOT overwrite.
      if (! projfile.exists())
        break;
      var ps = getPromptService();
      var promptTitle = gApp.getText("OverwriteProjectTitle");
      var promptMsg = gApp.getText("OverwriteProjectMsg");
      if (ps.confirm(window, promptTitle, promptMsg)) {
        return projdir;
      }
    }
    else
      break;
  }

  try {
    return bootstrapProject(saveConfig.location, saveConfig.title);
  }
  catch (ex) {
    dump("*** createProject: " + ex + "\n");
    return null;
  }
}


function bootstrapProject (dir, title) {
  var projdir = dir.clone();
  projdir.append(sanitizeFilename(title));
  projdir.createUnique(1, 0700);
  var celtxfile = projdir.clone();
  celtxfile.append(projdir.leafName + ".celtx");
  celtxfile.create(0, 0600);
  return projdir;
}


function showSplashWindow () {
  var med = Components.classes["@mozilla.org/appshell/window-mediator;1"]
    .getService(Components.interfaces.nsIWindowMediator);
  var splashes = med.getEnumerator("celtx:splash");
  if (splashes.hasMoreElements())
    splashes.getNext().focus();
  else
    window.openDialog(Cx.CONTENT_PATH + "templates.xul", "_blank",
                      "chrome,centerscreen,titlebar=no,border=no");
}



// update menu items that rely on focus
function goUpdateGlobalEditMenuItems() {
  goUpdateCommand('cmd_undo');
  goUpdateCommand('cmd_redo');
  goUpdateCommand('cmd_cut');
  goUpdateCommand('cmd_copy');
  goUpdateCommand('cmd_paste');
  goUpdateCommand('cmd_selectAll');
  goUpdateCommand('cmd_delete');
}

// update menu items that rely on the current selection
function goUpdateSelectEditMenuItems() {
  goUpdateCommand('cmd_cut');
  goUpdateCommand('cmd_copy');
  goUpdateCommand('cmd_paste');
  goUpdateCommand('cmd_delete');
  goUpdateCommand('cmd_selectAll');
}

// update menu items that relate to undo/redo
function goUpdateUndoEditMenuItems() {
  goUpdateCommand('cmd_undo');
  goUpdateCommand('cmd_redo');
}

// update menu items that depend on clipboard contents
function goUpdatePasteMenuItems() {
  goUpdateCommand('cmd_paste');
}

function openPreferences (paneID) {
  window.openDialog(Cx.CONTENT_PATH + "prefs.xul", "Preferences",
    "chrome,titlebar,toolbar,centerscreen,modal", paneID);
}

function inspectDOMDocument() {
  window.openDialog("chrome://inspector/content/", "_blank", 
                    "chrome,all,dialog=no", document);
}

function openJavaScriptConsole () {
  window.openDialog("chrome://global/content/console.xul", "_blank",
    "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar");
}


function authLoaded (request, service, callback) {
  dump("--- authLoaded\n");
  var authMgr = getAuthManager();

  if (request.status == 401) {
    // Not Authorized
    dump("401 not authorized\n");
    authMgr.clearAll();
    // Retry
    gApp.authenticate(service, callback, true);
  }
  else if (request.status != 200) {
    dump("error: status: " + request.status + "\n");
    authMgr.clearAll();
    callback(false);
  }
  else {
    callback(true);
  }
}


function authError (request, service, callback) {
  dump("--- authError\n");
  callback(false);
}
