/* ***** BEGIN LICENCE BLOCK *****
 * Version: CePL 1.1
 * 
 * The contents of this file are subject to the Celtx Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.celtx.com/CePL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is Celtx Script Manager.
 * 
 * The Initial Developer of the Original Code is Chad House and 4067479
 * Canada Inc. t/a CELTX.
 * 
 * Portions created by Chad House are Copyright (C) 2000-2006 Chad House,
 * parts created by Celtx are Copyright (C) 4067479 Canada Inc. All Rights
 * Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENCE BLOCK ***** */


function FormController (name) {
  this.formName = name;
  this.commands = {
    "cmd-page-setup": 1,
    "cmd-print": 1,
    "cmd-print-preview": 1,
  };
  this.openwindows = { findreplace: null, spellcheck: null };
}


FormController.prototype.supportsCommand = function (cmd) {
  return this.commands[cmd] == 1;
};


FormController.prototype.isCommandEnabled = function (cmd) {
  // XXX
  switch (cmd) {
    case "cmd-print-preview":
      return ! this.inPrintPreview;
    case "cmd-page-setup":
    case "cmd-print":
      return true;
    default:
      return false;
  }
};


FormController.prototype.doCommand = function (cmd) {
  // XXX
  switch (cmd) {
    case "cmd-page-setup":
      PrintUtils.showPageSetup();
      break;
    case "cmd-print":
      {
        var sheet = document.getElementById("sheet");
        var inputs = sheet.contentDocument.forms[0].elements;
        for (var i = 0; i < inputs.length; i++) {
          if (inputs[i].type!="radio") {
            // inputs[i].nextSibling.innerHTML=inputs[i].value;
            var text = sheet.contentDocument.createTextNode(inputs[i].value);
            var printdiv = inputs[i].nextSibling;
            if (printdiv.hasChildNodes() && printdiv.firstChild)
              printdiv.replaceChild(text, printdiv.firstChild);
            else
              printdiv.appendChild(text);
          }
        }
        PrintUtils.print();
      }   
      break;
    case "cmd-print-preview":
      PrintUtils.printPreview(onEnterPrintPreview, onExitPrintPreview);
      break;
  }
};


FormController.prototype.updateCommands = function () {
  for (var cmd in this.commands)
    goUpdateCommand(cmd);
  top.goUpdateCommand("cmd-print-preview");
};


FormController.prototype.open = function (project, docres) {
  this.project  = project;
  this.model    = new RDFModel(project.ds);
  this.docres   = RES(docres.Value);
  var itemres   = this.model.target(this.docres, PROP("dc:source"));
  // Check if we've already got a breakdown item association
  if (itemres) {
    this.res    = itemres;
    return;
  }
  var doctype   = this.model.target(this.docres, PROP("cx:doctype"));
  if (! doctype) {
    dump("*** Whoah! No doctype\n");
    this.res    = this.docres;
    return;
  }
  var title     = this.model.target(this.docres, PROP("dc:title"));
  var docsds    = getRDFService().GetDataSourceBlocking(Cx.DOCTYPES_URL);
  var docmodel  = new RDFModel(docsds);
  var category  = docmodel.target(doctype, PROP("cx:category"));
  if (category) {
    this.res    = RES(Cx.DOCUMENTS_URL + generateID());
    this.model.add(this.res, PROP("dc:title"), title);
    this.model.add(this.res, PROP("rdf:type"), category);
    this.model.add(this.docres, PROP("dc:source"), this.res);
  }
  else {
    dump("*** No cx:category for " + doctype.value + "\n");
    this.res    = this.docres;
  }
};


FormController.prototype.modified getter = function () {
  return this.form.modified;
};


FormController.prototype.save = function () {
  dump("save!\n");
  this.store();
  // this.populate();  // Re-populate
  var self = this;
  function delayedPopulate () { self.populate(); };
  setTimeout(delayedPopulate, 100);
};


FormController.prototype.store = function () {
  // if (! this.form.modified) return;
  try {
    for (var e in this.form.changedFields) {
      dump("store: changed: " + e + "\n");
      var vals = this.form.values(e);
      if (this.form.schema[e].type == 'media') {
        this.updateComplex(this.propMap[e], vals);
      }
      else {
        this.updateResource(this.propMap[e], vals);
      }
    }
  }
  catch (ex) {
    dump("store: " + ex + "\n");
  }

};


FormController.prototype.updateResource = function (prop, vals) {
  try {
    setCustomPropValues(this.model, this.res, RES(prop), vals);
  }
  catch (ex) {
    dump("updateResource: " + ex + "\n");
  }
};


FormController.prototype.updateComplex = function (prop, vals) {
  // For now, the only complex type is a media sequence

  try {
    var mediaSeq = this.model.target(this.res, RES(prop));
    if (! mediaSeq) {
      // Mint one
      mediaSeq = RES(this.project.mintURI());
      dump("adding mediaSeq: " + mediaSeq + "\n");
      this.model.add(this.res, RES(prop), mediaSeq);
    }

    var cont = null;
    if (this.model.isContainer(mediaSeq)) {
      cont = this.model.container(mediaSeq);
    }
    else {
      cont = this.model.makeSeq(mediaSeq);
    }

    var list, i;
    var left = {};

    list = cont.elements();
    for (i = 0; i < list.length; i++) {
      left[list[i].value] = 1;
    }

    var typeMap = {};
    typeMap['video'] = Cx.NS_CX + 'Video';
    typeMap['audio'] = Cx.NS_CX + 'Audio';
    typeMap['image'] = Cx.NS_CX + 'Image';

    var rec, res, leafName;
    for (i = 0; i < vals.length; i++) {
      rec = vals[i];
      if (! rec.uri) {
        // New item
        res = RES(this.project.mintURI());
        dump("adding new resource: [" + res + "]\n");
        this.model.add(res, PROP('rdf:type'), RES(typeMap[rec.type]));
        cont.append(res);

        leafName = this.importFile(rec.src);
        setLiteralProp(this.model, res, PROP('cx:localFile'), LIT(leafName));
      }
      else {
        res = RES(rec.uri);
        delete left[rec.uri];
      }

      setLiteralProp(this.model, res, PROP('dc:title'), LIT(rec.title));
    }

    // Delete leftovers
    for (var uri in left) {
      dump("removing resource [" + uri + "]\n");
      res = RES(uri);
      cont.remove(res);
      // Find and remove all properties of resource
      list = this.model.find(res, null, null);
      for (i = 0; i < list.length; i++) {
        this.model.remove(list[i][0], list[i][1], list[i][2]);
      }
    }

  }
  catch (ex) {
    dump("updateComplex: " + ex + "\n");
  }
};


FormController.prototype.importFile = function (src) {
  // TODO
  dump("importFile: " + src + "\n");

  // New implementation
  var srcFile = fileURLToFile(src);
  var dstFile = copyToUnique(srcFile, this.project.projectFolder,
    srcFile.leafName);
  return dstFile.leafName;

  // Old implementation
  var srcFile = fileURLToFile(src);
  var file = this.project.projectFolder;
  var name = safeFileName(srcFile.leafName);
  file.append(name);
  file.createUnique(0, 0600);
  dump("file: " + file.path + "\n");
  var persist = getWebBrowserPersist();
  persist.persistFlags |= persist.PERSIST_FLAGS_BYPASS_CACHE;
  // persist.progressListener = this;
  var srcURL = getIOService().newFileURI(srcFile);
  persist.saveURI(srcURL, null, null, null, null, file);
  return file.leafName;
};


FormController.prototype.close = function () {
  if (this.form.modified) {
    // TODO: prompt to save if modified
    this.store();
  }
};


FormController.prototype.focus = function () {
  var sheet = document.getElementById("sheet");
  if (! sheet)
    return;
  sheet.setAttribute("type", "content-primary");
};


FormController.prototype.blur = function () {
  if (this.inPrintPreview)
    PrintUtils.exitPrintPreview();
  var sheet = document.getElementById("sheet");
  if (! sheet)
    return;
  sheet.setAttribute("type", "content");
};


FormController.prototype.checkSpelling = function () {

};


FormController.prototype.findReplace = function (showReplace) {

};


FormController.prototype.findAgain = function (cmd) {

};


FormController.prototype.attach = function (form) {
  this.form = form;
  this.buildSchema();
};


FormController.prototype.setTitle = function (title) {
  try {
    var doc = this.form.contentDocument;
    var labels = doc.getElementsByTagName('itemlabel');
    var label = labels[0];
    if (! label) return;
    while (label.hasChildNodes()) {
      label.removeChild(label.lastChild);
    }
    label.appendChild(doc.createTextNode(title));
  }
  catch (ex) {
    dump(ex);
  }

};


FormController.prototype.buildSchema = function () {
  this.propMap = {};
  this.nameMap = {};

  // Description is Dublin Core
  this.nameMap[Cx.NS_DC + "description"] = "description";
  this.propMap["description"] = Cx.NS_DC + "description";

  // Comments and Media are global
  this.nameMap[Cx.NS_CX + "comments"] = "comments";
  this.propMap["comments"] = Cx.NS_CX + "comments";

  this.nameMap[Cx.NS_CX + "media"] = "media";
  this.propMap["media"] = Cx.NS_CX + "media";

  // cx:scriptName corresponds to script-name
  this.nameMap[Cx.NS_CX + "scriptName"] = "script-name";
  this.propMap["script-name"] = Cx.NS_CX + "scriptName";

  try {
    for (var name in this.form.schema) {
      var prop = this.customPropertyURI(name);
      if (! this.propMap[name]) {
        this.propMap[name] = prop;
        this.nameMap[prop] = name;
      }
    }
  }
  catch (ex) {
    dump("buildSchema: " + ex + "\n");
  }
};


FormController.prototype.customPropertyURI = function (name) {
  return this.project.res.Value + '/NS/' + this.formName + '-' + name;
};


FormController.prototype.populate = function () {
  try {
    var title = this.model.target(this.res, PROP('dc:title'));
    this.setTitle(title.value);

    var rec = this.fetch();
    this.form.fill(rec);
  }
  catch (ex) {
    dump("populate: " + rec + "\n");
  }
};


FormController.prototype.fetch = function () {
  var rec = {};
  var e, i, p, targets;

  for (e in this.propMap) {
    rec[e] = [];
    p = this.propMap[e];
    targets = this.model.targets(this.res, RES(p));
    for (i = 0; i < targets.length; i++) {
      if (targets[i].type == 'resource') {
        // There should be only one target
        rec[e] = this.fetchComplex(targets[i], this.form.schema[e].type);
      }
      else {
        rec[e].push(targets[i].value);
      }
    }
  }

  return rec;
};


FormController.prototype.fetchComplex = function (res, kind) {
  dump("fetchComplex: " + res + ", " + kind + "\n");

  // Only one type for now
  if (kind != 'media') return null;

  var typeMap = {};
  typeMap[Cx.NS_CX + 'Video'] = 'video';
  typeMap[Cx.NS_CX + 'Audio'] = 'audio';
  typeMap[Cx.NS_CX + 'Image'] = 'image';

  if (! this.model.isContainer(res)) {
    dump("fetchComplex: not a container\n");
    return [];
  }

  var rec, item, title, leaf, type;
  var items = [];
  var cont = this.model.container(res);
  var list = cont.elements();

  for (var i = 0; i < list.length; i++) {
    item = list[i];

    rec = { uri: item.value, title: '', src: '', type: '' };

    title = this.model.target(item, PROP('dc:title'));
    if (title) rec.title = title.value;

    leaf = this.model.target(item, PROP('cx:localFile'));
    if (leaf) rec.src = this.project.fileURLOf(leaf.value);

    type = this.model.target(item, PROP('rdf:type'));
    if (type) rec.type = typeMap[type.value];
    
    items.push(rec);
  }

  return items;
};


function onEnterPrintPreview () {
  var printPreviewTB = document.createElementNS(XUL_NS, "toolbar");
  printPreviewTB.setAttribute("printpreview", true);
  printPreviewTB.setAttribute("id", "print-preview-toolbar");
  getBrowser().parentNode.insertBefore(printPreviewTB, getBrowser());
  document.getElementById("formtoolbar").hidden = true;
  gController.inPrintPreview = true;
  gController.updateCommands();
  getBrowser().contentWindow.focus();
}


function onExitPrintPreview () {
  var printPreviewTB = document.getElementById("print-preview-toolbar");
  if (printPreviewTB)
    printPreviewTB.parentNode.removeChild(printPreviewTB);
  document.getElementById("formtoolbar").hidden = false;
  gController.inPrintPreview = false;
  gController.updateCommands();
}

